require 'enumerable_observer'
require 'active_support/cache'

module T1000
  class TransactionList < Array
    include EnumerableObserver

    alias_method :track_before, :dup

    def self.for_ip(ip, timeout: 3600)
      new(ip, timeout: timeout)
    end

    def initialize(ip, timeout: 3600)
      @timeout = timeout
      @key     = ActiveSupport::Cache.expand_cache_key [ip], :ip_request_tracker
      value    = T1000.cache.store.read(@key)
      replace Marshal.load value if value
      observe.added do
        T1000.cache.store.write @key, Marshal.dump(dup), expires_in: @timeout
      end
    rescue Exception => e
      T1000.cache.store.delete @key
      raise e
    end

    def within(time)
      dup.replace select { |t| t.time > time.ago }
    end

    def with_error
      dup.replace select { |t| t.response.status >= 400 }
    end

  end
end
